from abc import ABC, abstractmethod
from requests import Response

class Packet(ABC):
    ''' Represents an abstract data packet '''

    def __init__(self, ip, port, query):
        """Initialize packet
        
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5555.
            query (str): packet payload
        """
        self.ip = ip
        self.port = port
        self.query = query

    @abstractmethod
    def send(self, timeout=None):
        """Send packet abstract methud
        
        Args:
            timeout (int, optional): packet timeout. defaults to None.
        """
        pass

    @staticmethod
    def _generateResponse(content):
        """Generates response from packet
        
        Args:
            content (bytes): response content
        
        Returns:
            Response: structured response
        """
        response = Response()

        response._content = content
        response.status_code = 200

        return response

    @staticmethod
    def _generateEmptyResponse(errorType='unknown'):
        """Generates empty response packet
        
        Args:
            errorType (bytes, optional): error type. defaults to b'unknown'.
        
        Returns:
            Response: error response
        """
        response = Response()

        response._content = ('{ "exception": "Request resolved with an %s error" }' % errorType).encode()
        response.code = "expired"
        response.error_type = "expired"
        response.status_code = 400

        return response

    def __eq__(self, other):
        return isinstance(other, Packet) and self.ip == other.ip and self.port == other.port and self.query == other.query



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.md', encoding="utf8") as f:
    readme = f.read()

with open('LICENSE') as f:
    license = f.read()

setup(
    name='butter.mas-api',
    version='0.11.8',
    description='Python Client API for Butter MAS platform',
    long_description_content_type='text/markdown',
    long_description=readme,
    author='Benny Megidish',
    author_email='bennymegk@gmail.com',
    url='https://github.com/benymeg/Butter.MAS.PythonAPI',
    install_requires=['requests'],
    #license=license,
    license='Apache-2.0',
    packages=find_packages(exclude=('tests', 'docs'))
)

''' how to deploy '''
# python setup.py sdist bdist_wheel
# python -m twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# pip install --index-url https://test.pypi.org/simple/ --no-deps butter.mas-api

# python -m twine upload dist/*
# pip install -U butter.mas-api

''' how to generate documentation '''
# cd sphinx
# ./make html
# XCOPY ".\_build\html" "..\docs"


from .client import Client

class TcpClient(Client):
    ''' Butter MAS TCP client API '''

    def __init__(self, ip, port=5050, protocol='tcp'):
        """Initialize Butter MAS TCP client
                
        Args:
            ip (str): robot IP
            port (int, optional): robot port. Defaults to 5000.
            protocol (str, optional): communication protocol. Defaults to "tcp".
        """
        super().__init__(ip, port, protocol)

if __name__ == "__main__":
    client = TcpClient('localhost')

    print(client.getAvailableHandlers())